package org.jboss.ejb3.test.common.registrar.unit;

import org.jboss.beans.metadata.plugins.AbstractDependencyMetaData;
import org.jboss.beans.metadata.plugins.builder.BeanMetaDataBuilderFactory;
import org.jboss.beans.metadata.spi.builder.BeanMetaDataBuilder;
import org.jboss.dependency.plugins.AbstractDependencyItem;
import org.jboss.dependency.spi.ControllerMode;
import org.jboss.dependency.spi.ControllerState;
import org.jboss.dependency.spi.ErrorHandlingMode;
import org.jboss.ejb3.common.registrar.plugin.mc.Ejb3McRegistrar;
import org.jboss.ejb3.common.registrar.spi.Ejb3Registrar;
import org.jboss.ejb3.test.common.registrar.FakeMBean;
import org.jboss.ejb3.test.common.registrar.SimplePojo;
import org.jboss.kernel.Kernel;
import org.jboss.kernel.plugins.bootstrap.basic.BasicBootstrap;
import org.jboss.kernel.spi.dependency.KernelControllerContext;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertNull;

/**
 * Ensure we can use the Ejb3McRegistar from a START state.
 *
 * @author <a href="mailto:cdewolf@redhat.com">Carlo de Wolf</a>
 */
public class JBPAPP10454TestCase
{
   private static BasicBootstrap bootstrap;
   private static Ejb3Registrar registrar;

   @BeforeClass
   public static void beforeClass()
   {
      bootstrap = new BasicBootstrap();
      bootstrap.run();
      final Kernel kernel = bootstrap.getKernel();
      registrar = new Ejb3McRegistrar(kernel);
   }

   @Test
   public void testMBean() throws Throwable
   {
      {
         BeanMetaDataBuilder builder = BeanMetaDataBuilderFactory.createBuilder("SimplePojo", SimplePojo.class.getName());
         // Normally the whole batch of beans is installed and then it starts rumbling forward.
         // Now we do it in two steps, but make the first one on demand.
         builder.setMode(ControllerMode.ON_DEMAND);

         bootstrap.getKernel().getController().install(builder.getBeanMetaData());
      }

      {
         FakeMBean bean = new FakeMBean(registrar);
         BeanMetaDataBuilder builder = BeanMetaDataBuilderFactory.createBuilder("FakeMBean", FakeMBean.class.getName());
         // This we want to lookup in the start method.
         builder.addDependency("SimplePojo");

         KernelControllerContext context = bootstrap.getKernel().getController().install(builder.getBeanMetaData(), bean);
         assertNull("Failed with " + context.getError(), context.getError());
      }
   }
}
